
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_NeoPixel.h>

#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 32
#define LED_COUNT 5

//Pins
int ledstripUpper = 6;
int ledstripLower = 7;
int leftButton = 9;
int rightButton = 10;
int buzzer = 13;

//flow variables
bool started = false;
bool buttonPressLegal = true;

//Input variables
bool leftButtonIsPressed = false;
bool leftButtonWasPressedLastFrame = false;
bool rightButtonIsPressed = false;
bool rightButtonWasPressedLastFrame = false;

int ledInterval = 1000;
int toneLength = 250;
float lightFreq = 98.00                                                                                                                                                                       ;
float startFreq = 698.46;
int randomMin = 200;
int randomMax = 4000;
long timerStart = 0;

//display
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

//strips
Adafruit_NeoPixel upperStrip(LED_COUNT, ledstripUpper, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel lowerStrip(LED_COUNT, ledstripLower, NEO_GRB + NEO_KHZ800);


void setup()
{
  //For debug printing
  Serial.begin(9600);

  DisplaySetup();

  //setup the pins
  pinMode(buzzer, OUTPUT);
  pinMode(leftButton, INPUT_PULLUP);
  pinMode(rightButton, INPUT_PULLUP);


  upperStrip.begin();
  upperStrip.show();

  lowerStrip.begin();
  lowerStrip.show();

  ClearAllLeds();
}


void loop()
{
  ButtonInput();

  if (!started)
  {
    //Both buttons need to be pressed to start the game.
    //This way, both players are ready when the game starts.
    if (!digitalRead(leftButton) && !digitalRead(rightButton))
    {
      StartLights();
    }
  }
  else
  {
    if (LeftButtonDown())
    {
      StopTimer(0);
    }

    if (RightButtonDown())
    {
      StopTimer(1);
    }
  }
}

//Turns on the LED strips with an interval called "ledInterval"
void StartLights()
{
  started = true;
  buttonPressLegal = false;

  ClearDisplay();
  ClearAllLeds();

  for (int i = 0; i < LED_COUNT; i++)
  {
    //Turns upper and lower LED strip to red with interval of "ledInterval"
    
    tone(buzzer, lightFreq, toneLength);
    upperStrip.setPixelColor(i, 255, 0, 0, 0);
    lowerStrip.setPixelColor(i, 255, 0, 0, 0);
    upperStrip.show();
    lowerStrip.show();
    ButtonInput();

    if (i != LED_COUNT - 1)
    {
      delay(ledInterval);
    }
  }

  long waitTime = random(randomMin, randomMax); //Randomizes waittime so that the player cannot predict when the lights will turn off.

  Serial.print("Waittime: ");
  Serial.println(waitTime);

  delay(waitTime);

  ButtonInput();
  buttonPressLegal = true;
  timerStart = millis();
  tone(buzzer, startFreq, toneLength * 3);
  ClearAllLeds();
}

//Stops the reaction time timer.
//buttonIndex is the number of the button that was pressed. This determines which player has won.
void StopTimer(int buttonIndex)
{
  if (buttonPressLegal)
  {
    switch (buttonIndex)
    {
      case 0:
        //left won. Show green light on the left of upper strip.
        upperStrip.setPixelColor(0, 0, 255, 0, 0);
        upperStrip.show();
        break;
      case 1:
        //right won. Show green light on the right of upper strip.
        upperStrip.setPixelColor(LED_COUNT - 1, 0, 255, 0, 0);
        upperStrip.show();
        break;
    }

    //the reaction time is the difference between now and when the timer started.
    long reactionTime = millis() - timerStart;
    ShowReactionTime((float)reactionTime / 1000); //reaction time is stated in milliseconds but we need seconds so divide by 1000
  }

  delay(1000);
  started = false;
}

//Turns off lower and upper LED strips.
void ClearAllLeds()
{
  upperStrip.clear();
  lowerStrip.clear();

  upperStrip.show();
  lowerStrip.show();
}

//Initializes the OLED display.
void DisplaySetup()
{
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3D for 128x64
    Serial.println(F("SSD1306 allocation failed"));
    for (;;);
  }
  delay(100);
  ClearDisplay();
}

void ClearDisplay()
{
  display.clearDisplay();
  display.display();
}

//Show the reaction time number on the OLED display.
// Number is rounded to 3 decimals.
void ShowReactionTime(float reactionTime)
{
  String reactionTimeText = String(reactionTime, 3);

  display.setTextSize(4);
  display.setTextColor(WHITE);
  display.setCursor(5, 2);
  display.println(reactionTimeText);
  display.display();
}

//Returns true during the tick the user pressed the left button.
bool LeftButtonDown()
{
  return leftButtonIsPressed && !leftButtonWasPressedLastFrame;
}

//Returns true during the tick the user pressed the right button.
bool RightButtonDown()
{
  return rightButtonIsPressed && !rightButtonWasPressedLastFrame;
}

//Makes sure the buttons cannot be held down.
void ButtonInput()
{
  leftButtonWasPressedLastFrame = leftButtonIsPressed;
  rightButtonWasPressedLastFrame = rightButtonIsPressed;

  leftButtonIsPressed = !digitalRead(leftButton);
  rightButtonIsPressed = !digitalRead(rightButton);
}
